
import codecs
import os.path

import wx
import wx.html
from wx.lib.scrolledpanel import ScrolledPanel

import build
import upload
import dictionary

class UploadPanel(ScrolledPanel):
    """Holds checkboxes and login info for upload"""

    def __init__(self, *args, **kwargs):
        ScrolledPanel.__init__(self, *args, **kwargs)

        sizer = wx.BoxSizer(wx.VERTICAL)

        self.checkboxes = []
        for root, dirs, files in os.walk("output"):
            for name in files:
                # ignore hidden files
                if name.startswith("."):
                    continue

                # add a checkbox for each file (ignoring "output/")
                path = name
                splitroot = root.split(os.sep)[1:]
                if len(splitroot) != 0:
                    path = os.path.join(os.path.join(*root.split(os.sep)[1:]), name)

                item = wx.CheckBox(self, wx.ID_ANY, path)
                item.SetValue(path.endswith(".html"))
                self.checkboxes.append(item)
                sizer.Add(item)

        # make the panel scroll
        self.SetSizer(sizer)
        self.SetupScrolling()

    def selectedpaths(self):
        for item in self.checkboxes:
            if item.IsChecked():
                yield item.GetLabelText()



class UploadDialog(wx.Dialog):
    """Create dialog for uploading files."""

    def __init__(self, *args, **kwargs):
        wx.Dialog.__init__(self, *args, **kwargs)

        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        # create login box
        loginsizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.AddSizer(loginsizer)
        sizer.AddSpacer(4)

        # get address to ftp server
        self.path = wx.TextCtrl(self, value="ftp.example.com")
        loginsizer.Add(self.path)
        loginsizer.AddSpacer(4)

        # get login from file
        user = "<username>"
        if os.path.isfile("user.txt"):
            f = open("user.txt")
            user = f.read().strip()
            f.close()
        self.username = wx.TextCtrl(self, value=user)
        loginsizer.Add(self.username)
        loginsizer.AddSpacer(4)

        self.password = wx.TextCtrl(self, style=wx.TE_PASSWORD)
        loginsizer.Add(self.password)
        loginsizer.AddSpacer(4)

        upload = wx.Button(self, label="Login and Upload")
        upload.Bind(wx.EVT_BUTTON, self.event_upload)
        loginsizer.Add(upload)

        # create file selector
        self.filechooser = UploadPanel(self)
        sizer.Add(self.filechooser, 1, wx.EXPAND)

    def event_upload(self, evt):
        """Uploads to the website (or displays dialog on failure"""

        try:
            upload.upload(self.username.GetValue(), self.password.GetValue(),
                    self.path.GetValue(), self.filechooser.selectedpaths())
            self.Close()

        except upload.LoginError:
            dialog = wx.MessageDialog(self, "Error loggging in.",
                    "Login Error", style=wx.OK)
            dialog.ShowModal()
            dialog.Destroy()

        except upload.UnknownError:
            dialog = wx.MessageDialog(self, "Error occurred while uploading.",
                    "Upload Error", style=wx.OK)
            dialog.ShowModal()
            dialog.Destroy()



class EditPanel(wx.Panel):

    def __init__(self, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)

        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        topsizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.AddSizer(topsizer)

        # allow choosing different pages
        navigation, self.pages = dictionary.load("pages.txt")
        page_choices = self.pages.keys()
        self.page_choice = wx.Choice(self, wx.ID_ANY, choices=page_choices)
        self.page_choice.Bind(wx.EVT_CHOICE, self.event_pagechanged)
        topsizer.Add(self.page_choice)

        # button for exporting website
        exportbutton = wx.Button(self, wx.ID_ANY, "Upload Website")
        exportbutton.Bind(wx.EVT_BUTTON, self.event_export)
        topsizer.Add(exportbutton)

        # text box for editing html
        self.html_editor = wx.TextCtrl(self, wx.ID_ANY,
                style=wx.TE_MULTILINE|wx.TE_PROCESS_TAB)
        self.html_editor.Bind(wx.EVT_TEXT, self.event_htmledited)
        sizer.Add(self.html_editor, 1, wx.EXPAND)

        # html viewer for displaying pages
        self.html_viewer = wx.html.HtmlWindow(self, wx.ID_ANY)
        sizer.Add(self.html_viewer, 1, wx.EXPAND)

        self.timer = wx.Timer(self, wx.ID_ANY)
        self.Bind(wx.EVT_TIMER, self.dispatch_save, self.timer)
        self.loadpage(page_choices[0])


    def event_pagechanged(self, evt):
        self.loadpage(self.page_choice.GetStringSelection())

    def event_htmledited(self, evt):
        if self.timer.IsRunning():
            self.timer.Stop()
        self.timer.Start(500, wx.TIMER_ONE_SHOT)

    def event_export(self, evt):
        """exports the website, backs up, and uploads"""
        # upload.backup() -- this will probably just confuse her...
        build.build()

        # make a dialog for uploading to FTP
        dialog = UploadDialog(self, wx.ID_ANY,
                style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
        dialog.Center()
        dialog.ShowModal()
        dialog.Destroy()

    def dispatch_save(self, evt):
        """Saves the currently editted text to a file."""
        self.html_viewer.SetPage(self.html_editor.GetValue())
        self.savepage(self.page_choice.GetStringSelection())

    def savepage(self, page):
        """Saves page to a file"""
        f = codecs.open(os.path.join("pages", page), "w", "utf-8")
        f.write(self.html_editor.GetValue())
        f.close()

    def loadpage(self, page):
        """Loads a page file into editor"""
        pagepath = os.path.join("pages", page)

        # load the text from the page file
        # create the file if it does not exist
        text = ""
        if os.path.isfile(pagepath):
            f = codecs.open(pagepath, "r", "utf-8")
            text = f.read()
            f.close()
        else:
            f = open(pagepath, "w")
            f.close()

        self.html_editor.ChangeValue(text)
        self.html_viewer.SetPage(text)



class EditFrame(wx.Frame):

    def __init__(self, *args, **kwargs):
        """Create a frame for editing html."""
        wx.Frame.__init__(self, *args, **kwargs)

        self.Center()
        panel = EditPanel(self, wx.ID_ANY)



if __name__ == "__main__":
    app = wx.PySimpleApp()
    frame = EditFrame(None, wx.ID_ANY)
    frame.Show(True)

    wx.InitAllImageHandlers()
    app.MainLoop()

